@@ -38,7 +38,7 @@ resNode			*ResNodeQueue = NULL;	/* Pending nodes	  */
 resNode			*ResOriginNode = NULL;	/* node where R=0	  */
 resNode			*resCurrentNode;
 int			ResTileCount = 0;	/* Number of tiles rn_status */
-extern Region 		*ResFirst();
+extern ExtRegion 	*ResFirst();
 extern Tile		*FindStartTile();
 extern int		ResEachTile();
 extern int		ResLaplaceTile();
@@ -132,9 +132,8 @@ ResGetReCell()
  *  results:  none
  *
  *  Side Effects:  All contacts in the design are broken into their
- *    constituent
- *    layers.  There should be no contacts in ResDef after this procedure
- *    runs.
+ *    constituent layers.  There should be no contacts in ResDef after
+ *    this procedure runs.
  *
  *
  *------------------------------------------------------------------------
@@ -147,7 +146,7 @@ ResDissolveContacts(contacts)
     Tile *tp;
     TileTypeBitMask residues;
 
-    for (; contacts != (ResContactPoint *) NULL; contacts = contacts->cp_nextcontact)
+    for (; contacts != (ResContactPoint *)NULL; contacts = contacts->cp_nextcontact)
     {
         oldtype=contacts->cp_type;
 
@@ -224,7 +223,6 @@ ResMakePortBreakpoints(def)
 		rect->r_ytop++;
 	    }
 
-
 	    /* If label is on a contact, the contact has been dissolved. */
 	    /* Assume that the uppermost residue is the port.  This may	 */
 	    /* not necessarily be the case.  Could do a boundary scan on */
@@ -869,6 +867,61 @@ resExpandDevFunc(tile, cx)
     return 1;
 }
 
+/*
+ *-------------------------------------------------------------------------
+ *
+ * ResShaveContacts ---
+ *
+ *	Remove the top layer off of every contact in the design, leaving
+ *	only the bottom layer.  This also resolves issues with stacked
+ *	contacts by leaving clean contact areas where stacked types
+ *	overlap.  Contacts are removed from the plane above the search
+ *	plane, so the removal does not corrupt the current plane search.
+ *
+ * Results:
+ *	Return 0 to keep the search going.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+int
+ResShaveContacts(tile, def)
+    Tile *tile;
+    CellDef *def;
+{
+    TileType ttype;
+    TileTypeBitMask *rmask;
+    Rect area;
+    Plane *plane;
+    int pNum;
+    int pMask;
+
+    /* To do:  Handle split tiles, although this is unlikely for
+     * contact types.
+     */
+    ttype = TiGetType(tile);
+
+    if (DBIsContact(ttype))
+    {
+	/* Remove the contact type from the plane above */
+	TiToRect(tile, &area);
+	pMask = DBTypePlaneMaskTbl[ttype];
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+	    if (PlaneMaskHasPlane(pMask, pNum))
+		break;
+
+	for (++pNum; pNum < DBNumPlanes; pNum++)
+	    if (PlaneMaskHasPlane(pMask, pNum))
+	    {
+		plane = def->cd_planes[pNum];
+		DBPaintPlane(plane, &area, DBStdEraseTbl(ttype, pNum),
+			(PaintUndoInfo *)NULL);
+	    }
+    }
+
+    return 0;
+}
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -1016,16 +1069,12 @@ ResExtractNet(node, goodies, cellname)
 	/* the device identifier type, then add the source/drain types to   */
 	/* the mask ResSDTypesBitMask.					    */
 
-	for (devptr = ExtCurStyle->exts_device[thisDev->type]; devptr;
-		    devptr = devptr->exts_next)
-	    for (i = 0; !TTMaskIsZero(&devptr->exts_deviceSDTypes[i]); i++)
-		TTMaskSetMask(&ResSDTypesBitMask, &devptr->exts_deviceSDTypes[i]);
+	devptr = tptr->thisDev->rs_devptr;
+	for (i = 0; !TTMaskIsZero(&devptr->exts_deviceSDTypes[i]); i++)
+	    TTMaskSetMask(&ResSDTypesBitMask, &devptr->exts_deviceSDTypes[i]);
 
 	/* Add the substrate types to the mask ResSubTypesBitMask	    */
-
-	for (devptr = ExtCurStyle->exts_device[thisDev->type]; devptr;
-		    devptr = devptr->exts_next)
-	    TTMaskSetMask(&ResSubTypesBitMask, &devptr->exts_deviceSubstrateTypes);
+	TTMaskSetMask(&ResSubTypesBitMask, &devptr->exts_deviceSubstrateTypes);
 
 	/* TT_SPACE should be removed from ResSubTypesBitMask */
 	TTMaskClearType(&ResSubTypesBitMask, TT_SPACE);
@@ -1034,8 +1083,28 @@ ResExtractNet(node, goodies, cellname)
 
     ExtResetTiles(scx.scx_use->cu_def, extUnInit);
 
+    /* To avoid issues with overlapping stacked contact types and	*/
+    /* double-counting contacts on multiple planes, erase the top	*/
+    /* contact layers of all contacts.  ExtFindRegions() will still	*/
+    /* find the connectivity above but will only process one tile per	*/
+    /* contact.	  This temporarily creates an improper database, but	*/
+    /* the contacts are all immediately erased by ResDissolveContacts().*/
+
+    for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+    {
+	Plane *plane = ResUse->cu_def->cd_planes[pNum];
+	DBSrPaintArea(plane->pl_hint, plane, &(ResUse->cu_def->cd_bbox),
+		&DBAllButSpaceAndDRCBits, ResShaveContacts,
+		(ClientData)ResUse->cu_def);
+    }
+
     /* Find all contacts in design and note their position */
 
+    /* NOTE:  ExtFindRegions() will call ResFirst or ResEach for BOTH	*/
+    /* planes of a contact.  Rather than attempting to limit the	*/
+    /* search, ResDoContacts() will just double the resistance per via	*/
+    /* so that the final value is correct.				*/
+
     ResContactList = (ResContactPoint *)ExtFindRegions(ResUse->cu_def,
 				     &(ResUse->cu_def->cd_bbox),
 				     &DBAllButSpaceAndDRCBits,
@@ -1162,11 +1231,42 @@ ResCleanUpEverything()
     DBCellClearDef(ResUse->cu_def);
 }
 
+/*
+ *-------------------------------------------------------------------------
+ *
+ * ResGetTileFunc --
+ *
+ *	Callback function used by FindStartTile() when searching for
+ *	terminal connections of a device that may be on planes other
+ *	than the plane of the device identifier type.  Ignore space
+ *	tiles.  Otherwise, for any tile found, record the tile in
+ *	the client data record and return 1 to stop the search.
+ *
+ * Results:
+ *	Return 0 if tile is a space tile, to keep the search going.
+ *	Return 1 otherwise to stop the search immediately because
+ *	a valid start tile has been found.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+int
+ResGetTileFunc(tile, tpptr)
+    Tile *tile, **tpptr;
+{
+    if (TiGetType(tile) != TT_SPACE)
+    {
+	*tpptr = tile;
+	return 1;
+    }
+    return 0;
+}
+
 /*
  *-------------------------------------------------------------------------
  *
  * FindStartTile-- To start the extraction, we need to find the first driver.
- *	The sim file gives us the location of a point in  or near (within 1
+ *	The sim file gives us the location of a point in or near (within 1
  *	unit) of the device. FindStartTile looks for the device, then
  *	for adjoining diffusion. The diffusion tile is returned.
  *
@@ -1186,8 +1286,9 @@ FindStartTile(goodies, SourcePoint)
 {
     Point	workingPoint;
     Tile	*tile, *tp;
-    int		pnum, t1, t2;
+    int		pnum, t1, t2, i;
     ExtDevice   *devptr;
+    Rect	r;
 
     /* If the drive point is on a contact, check for the contact residues   */
     /* first, then the contact type itself.				    */
@@ -1282,59 +1383,91 @@ FindStartTile(goodies, SourcePoint)
     else
 	t1 = TiGetType(tile);
 
-    devptr = ExtCurStyle->exts_device[t1];
+    /* NOTE: There must be a way to pass the device type from a device
+     * record's rs_devptr instead of groping around for it.
+     */
 
-    /* left */
-    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
+    for (devptr = ExtCurStyle->exts_device[t1]; devptr; devptr = devptr->exts_next)
     {
-	t2 = TiGetRightType(tp);
-	if (TTMaskHasType(&(devptr->exts_deviceSDTypes[0]), t2))
+	for (i = 0; i < devptr->exts_deviceSDCount; i++)
 	{
-	    SourcePoint->p_x = LEFT(tile);
-	    SourcePoint->p_y = (MIN(TOP(tile),TOP(tp)) +
+	    /* left */
+	    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
+	    {
+		t2 = TiGetRightType(tp);
+		if ((t2 != TT_SPACE) &&
+			TTMaskHasType(&(devptr->exts_deviceSDTypes[i]), t2))
+		{
+		    SourcePoint->p_x = LEFT(tile);
+		    SourcePoint->p_y = (MIN(TOP(tile),TOP(tp)) +
 		   			MAX(BOTTOM(tile), BOTTOM(tp))) >> 1;
-	    return(tp);
-	}
-    }
+		    return(tp);
+		}
+	    }
 
-    /* right */
-    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
-    {
-	t2 = TiGetLeftType(tp);
-	if (TTMaskHasType(&(devptr->exts_deviceSDTypes[0]), t2))
-	{
-	    SourcePoint->p_x = RIGHT(tile);
-	    SourcePoint->p_y = (MIN(TOP(tile), TOP(tp))+
+	    /* right */
+	    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
+	    {
+		t2 = TiGetLeftType(tp);
+		if ((t2 != TT_SPACE) &&
+			TTMaskHasType(&(devptr->exts_deviceSDTypes[i]), t2))
+		{
+		    SourcePoint->p_x = RIGHT(tile);
+		    SourcePoint->p_y = (MIN(TOP(tile), TOP(tp))+
 		   			MAX(BOTTOM(tile), BOTTOM(tp))) >> 1;
-	    return(tp);
+		    return(tp);
+		}
+	    }
+
+	    /* top */
+	    for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp = BL(tp))
+	    {
+		t2 = TiGetBottomType(tp);
+		if ((t2 != TT_SPACE) &&
+			TTMaskHasType(&(devptr->exts_deviceSDTypes[i]), t2))
+		{
+		    SourcePoint->p_y = TOP(tile);
+		    SourcePoint->p_x = (MIN(RIGHT(tile),RIGHT(tp)) +
+		   			MAX(LEFT(tile), LEFT(tp))) >> 1;
+		    return(tp);
+		}
+	    }
+
+	    /* bottom */
+	    for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp = TR(tp))
+	    {
+		t2 = TiGetTopType(tp);
+		if ((t2 != TT_SPACE) &&
+			TTMaskHasType(&(devptr->exts_deviceSDTypes[i]), t2))
+		{
+		    SourcePoint->p_y = BOTTOM(tile);
+		    SourcePoint->p_x = (MIN(RIGHT(tile), RIGHT(tp)) +
+		   			MAX(LEFT(tile), LEFT(tp))) >> 1;
+		    return(tp);
+		}
+	    }
+	}
+
+	/* Didn't find a terminal (S/D) type tile in the perimeter search.	*/
+	/* Check if S/D types are in a different plane from the identifier.	*/
+
+	TiToRect(tile, &r);
+	tp = NULL;
+	for (i = 0; i < devptr->exts_deviceSDCount; i++)
+	{
+	    for (pnum = 0; pnum < DBNumPlanes; pnum++)
+	    {
+		DBSrPaintArea((Tile *)NULL, ResUse->cu_def->cd_planes[pnum],
+			&r, &(devptr->exts_deviceSDTypes[i]), ResGetTileFunc, &tp);
+		if (tp != NULL) return tp;
+	    }
 	}
     }
 
-    /* top */
-    for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp = BL(tp))
-    {
-	t2 = TiGetBottomType(tp);
-	if (TTMaskHasType(&(devptr->exts_deviceSDTypes[0]), t2))
-	{
-	    SourcePoint->p_y = TOP(tile);
-	    SourcePoint->p_x = (MIN(RIGHT(tile),RIGHT(tp)) +
-		   			MAX(LEFT(tile), LEFT(tp))) >> 1;
-	    return(tp);
-	}
-    }
+    /* Didn't find a terminal (S/D) type tile anywhere.  Flag an error. */
 
-    /* bottom */
-    for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp = TR(tp))
-    {
-	t2 = TiGetTopType(tp);
-	if (TTMaskHasType(&(devptr->exts_deviceSDTypes[0]), t2))
-	{
-	    SourcePoint->p_y = BOTTOM(tile);
-	    SourcePoint->p_x = (MIN(RIGHT(tile), RIGHT(tp)) +
-		   			MAX(LEFT(tile), LEFT(tp))) >> 1;
-	    return(tp);
-	}
-    }
+    TxError("Couldn't find a terminal of the device at %d %d\n",
+			goodies->rg_devloc->p_x, goodies->rg_devloc->p_y);
     return((Tile *) NULL);
 }
 
