@@ -3,21 +3,21 @@
  *
  * Commands for the route modules only.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/router/rtrCmd.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/router/rtrCmd.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -45,6 +45,17 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/rout
 #include "gcr/gcr.h"
 #include "grouter/grouter.h"
 
+/* C99 compat */
+#include "textio/textio.h"
+#include "irouter/irouter.h"
+#include "mzrouter/mzrouter.h"
+#include "garouter/garouter.h"
+#include "router/routerInt.h"
+
+/* Global variable initialization */
+
+bool RtrMazeStems = FALSE;	/* Set by default to original behavior */
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -357,6 +368,8 @@ badWarn:
 		    return;
 		if (cmd->tx_argc == 3)
 		    channame = cmd->tx_argv[2];
+		else
+		    chanType = CHAN_NORMAL;	/* default */
 	    }
 	    else if (cmd->tx_argc == 6 || cmd->tx_argc == 7)
 	    {
@@ -479,6 +492,7 @@ CmdIRoute(w, cmd)
 #define ROUTERTECHINFO	16
 #define ROUTERVIAS	17
 #define	ROUTERVIAMIN	18
+#define	ROUTERMAZESTEMS	19
 
 void
 CmdRoute(w, cmd)
@@ -492,27 +506,28 @@ CmdRoute(w, cmd)
     char *netListName;
 
     static char *cmdRouteOption[] =
-    {	
+    {
 	"*checks		toggle column checking",
 	"*debug		toggle router trace messages",
-	"*file file		route from a channel data file",
+	"*file <file>		route from a channel data file",
 	"*map			toggle channel obstacle map display",
 	"*showcolumns		toggle channel column text display",
 	"*showfinal		toggle text display of final channels",
 
-	"end real		set channel router end constant",
+	"end <real>		set channel router end constant",
 	"help			print this help information",
 	"jog int		set minimum jog length (grid units)",
 	"metal			toggle metal maximization",
-	"netlist file		set current netlist",
-	"obstacle real		set obstacle constant",
+	"netlist <file>		set current netlist",
+	"obstacle <real>	set obstacle constant",
 	"origin [x y]		print or set routing grid origin",
 	"stats			print and clear previous route statistics",
-	"settings		show router parameters", 
-	"steady int		set steady net constant",
+	"settings		show router parameters",
+	"steady <int>		set steady net constant",
 	"tech			show router technology information",
-	"vias int		set metal maximization via limit (grid units)",
+	"vias <int>		set metal maximization via limit (grid units)",
 	"viamin			via minimization",
+	"mazestems		use maze router to route channel stems to pins",
 	NULL
     };
 
@@ -571,7 +586,7 @@ CmdRoute(w, cmd)
 	    TxPrintf("Router commands have the form \"route option\",\n");
 	    TxPrintf("where option is one of:\n\n");
 	    if(cmd->tx_argc==2)
-		for (option=ROUTEREND; option<= ROUTERVIAMIN; option++)
+		for (option=ROUTEREND; option<= ROUTERMAZESTEMS; option++)
 		    TxPrintf("  %s\n", cmdRouteOption[option]);
 	    else
 	    if((cmd->tx_argc==3) && !strcmp(cmd->tx_argv[2], "*wizard"))
@@ -591,6 +606,7 @@ CmdRoute(w, cmd)
 	    TxPrintf("   Obstacle constant...... %f\n", GCRObstDist);
 	    TxPrintf("   Steady net constant.... %d\n", GCRSteadyNet);
 	    TxPrintf("   Via limit.............. %d\n", RtrViaLimit);
+	    TxPrintf("   Maze route stems....... %s\n", RtrMazeStems ? "yes" : "no");
 	    if((cmd->tx_argc!=3) || strcmp(cmd->tx_argv[2], "*wizard"))
 		break;
 	    TxPrintf("\n");
@@ -714,6 +730,10 @@ CmdRoute(w, cmd)
 		NLFree(&netList);
 	    }
 	    break;
+	case ROUTERMAZESTEMS:
+	    RtrMazeStems = !RtrMazeStems;
+	    TxPrintf("Maze route channel stems: %s\n", RtrMazeStems ? "yes" : "no");
+	    break;
     } /* switch*/
     return;
 
